/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (c) 2005,2006 Sven Slezak <sunny@mezzo.net>
 */

/*! \file
 *
 * \brief Charset conversion related dialplan functions
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#ifndef BUILTIN_FUNC
#include <asterisk/module.h>
#endif /* BUILTIN_FUNC */
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/logger.h>
#include <asterisk/utils.h>
#include <asterisk/app.h>
#include <asterisk/options.h>

#include <ctype.h>
#include <errno.h>
#include <iconv.h>


int strconvert(const char *incharset, const char *outcharset,
#ifdef __Darwin__
               const char *in,
#else
               char *in,
#endif
               char *out) {
        iconv_t cd;
        size_t incount = strlen(in), outcount = strlen(in)*2, result;
        cd = iconv_open(outcharset, incharset);
        if(cd == (iconv_t) -1) {
                ast_log(LOG_ERROR, "conversion from '%s' to '%s' not available. type 'iconv -l' in a shell to list the supported charsets.\n", incharset, outcharset);
                memcpy(out, in, strlen(in)+1);
                return -1;
        }
        result = iconv(cd, &in, &incount, &out, &outcount);
        if(result == (size_t)-1) {
                if(errno == E2BIG) {
                        ast_log(LOG_WARNING, "Iconv: output buffer too small.\n");
                } else if(errno == EILSEQ) {
                        ast_log(LOG_WARNING,  "Iconv: illegal character.\n");
                } else if(errno == EINVAL) {
                        ast_log(LOG_WARNING,  "Iconv: incomplete character sequence.\n");
                } else {
                        ast_log(LOG_WARNING,  "Iconv: error.\n");
                }
        }
        iconv_close(cd);
        *out = '\0';
        return 1;
}


static char *iconv_read(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
{
  char *argv, *in_charset, *out_charset, *string, *string_converted;
  int arglen;
  buf[0] = '\0';

	if(!data) {
		ast_log(LOG_ERROR, "Asterisk function ICONV() requires arguments.\n");
		return buf;
	}
  arglen = strlen(data)+1;
  argv = alloca(arglen);
  if(!argv) {
     ast_log (LOG_ERROR, "Memory allocation failed\n");
     return buf;
  }
  memcpy (argv, data, arglen + 1);
	
	out_charset = ast_strip(strsep(&argv, "|"));
	in_charset = ast_strip(strsep(&argv, "|"));
	string = ast_strip(strsep(&argv, "\0"));

	if(ast_strlen_zero(in_charset) || ast_strlen_zero(out_charset) || ast_strlen_zero(string)) {
		ast_log(LOG_ERROR, "missing arguments ICONV(out-charset,in-charset,text).\n");
		return buf;
	}
  string_converted = malloc(sizeof(char)*strlen(string)*4);
  if(option_verbose > 3)
    ast_verbose(VERBOSE_PREFIX_3 "Iconv: \"%s\" %s -> %s\n", string, in_charset, out_charset);
  strconvert(in_charset, out_charset, string, string_converted);
  ast_copy_string(buf, string_converted, len);
  free(string_converted);
	return buf;
}

#ifndef BUILTIN_FUNC
static
#endif /* BUILTIN_FUNC */
struct ast_custom_function iconv_function = {
	.name = "ICONV",
	.synopsis = "Converts charsets of strings.",
	.syntax = "ICONV(out-charset,in-charset,string)",
	.desc = "Converts string from in-charset into out-charset.\n"
	"for available charsets call 'iconv -l' on your shell command line.\n",
	.read = iconv_read,
};

#ifndef BUILTIN_FUNC
static char *tdesc = "String charset conversion dialplan function";

int unload_module(void)
{
        return ast_custom_function_unregister(&iconv_function);
}

int load_module(void)
{
        return ast_custom_function_register(&iconv_function);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	return 0;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
#endif /* BUILTIN_FUNC */

/*
Local Variables:
mode: C
c-file-style: "linux"
indent-tabs-mode: nil
End:
*/
